@@ -4,31 +4,31 @@ use term::{Term, Notation, abs, app};
 use term::Term::*;
 use self::Token::*;
 use self::CToken::*;
-use self::Error::*;
+use self::ParseError::*;
 use self::Expression::*;
 pub use term::Notation::*;
 
-/// A type to represent a parsing error.
+/// An error returned by `parse()` when a parsing issue is encountered.
 #[derive(Debug, PartialEq)]
-pub enum Error {
-    /// an invalid character was encountered
+pub enum ParseError {
+    /// lexical error; contains the invalid character and its index
     InvalidCharacter((usize, char)),
-    /// the expression is invalid
+    /// syntax error; the expression is invalid
     InvalidExpression,
-    /// the expression is empty
+    /// syntax error; the expression is empty
     EmptyExpression
 }
 
 #[derive(Debug, PartialEq)]
 #[doc(hidden)]
 pub enum Token {
-    /// λ or \
+    /// the lambda symbol ('λ' or '\')
     Lambda,
     /// left parenthesis
     Lparen,
     /// right parenthesis
     Rparen,
-    /// a number
+    /// a hex-encoded digit
     Number(usize)
 }
 
@@ -46,7 +46,7 @@ pub enum CToken {
 }
 
 #[doc(hidden)]
-pub fn tokenize_dbr(input: &str) -> Result<Vec<Token>, Error> {
+pub fn tokenize_dbr(input: &str) -> Result<Vec<Token>, ParseError> {
     let chars = input.chars().enumerate();
     let mut tokens = Vec::new();
 
@@ -61,7 +61,7 @@ pub fn tokenize_dbr(input: &str) -> Result<Vec<Token>, Error> {
                 } else if c.is_whitespace() {
                     ()
                 } else {
-                    return Err(InvalidCharacter((i + 1, c)))
+                    return Err(InvalidCharacter((i, c)))
                 }
             }
         }
@@ -71,7 +71,7 @@ pub fn tokenize_dbr(input: &str) -> Result<Vec<Token>, Error> {
 }
 
 #[doc(hidden)]
-pub fn tokenize_cla(input: &str) -> Result<Vec<CToken>, Error> {
+pub fn tokenize_cla(input: &str) -> Result<Vec<CToken>, ParseError> {
     let mut chars = input.chars().enumerate().peekable();
     let mut tokens = Vec::new();
     let valid_chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
@@ -86,7 +86,7 @@ pub fn tokenize_cla(input: &str) -> Result<Vec<CToken>, Error> {
                     } else if valid_chars.contains(c) {
                         name.push(c)
                     } else {
-                        return Err(InvalidCharacter((i + 1, c)))
+                        return Err(InvalidCharacter((i, c)))
                     }
                 }
                 tokens.push(CLambda(name))
@@ -108,7 +108,7 @@ pub fn tokenize_cla(input: &str) -> Result<Vec<CToken>, Error> {
                     }
                     tokens.push(CName(name))
                 } else {
-                    return Err(InvalidCharacter((i + 1, c)))
+                    return Err(InvalidCharacter((i, c)))
                 }
             }
         }
@@ -163,18 +163,18 @@ fn _convert_classic_tokens(tokens: &[CToken], stack: &mut Vec<String>, pos: &mut
 pub enum Expression {
     /// an abstraction
     Abstraction,
-    /// a sequence of Expressions
+    /// a sequence of `Expression`s
     Sequence(Vec<Expression>),
     /// a variable with a De Bruijn index
     Variable(usize)
 }
 
 #[doc(hidden)]
-pub fn get_ast(tokens: &[Token]) -> Result<Expression, Error> {
+pub fn get_ast(tokens: &[Token]) -> Result<Expression, ParseError> {
     _get_ast(tokens, &mut 0)
 }
 
-fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
+fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, ParseError> {
     if tokens.is_empty() { return Err(EmptyExpression) }
 
     let mut expr = Vec::new();
@@ -202,27 +202,31 @@ fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
     Ok(Sequence(expr))
 }
 
-/// Parses the input `&str` as a lambda `Term`.
+/// Attempts to parse the input `&str` as a lambda `Term` encoded in the given `Notation`.
 ///
 /// - lambdas can be represented either with the greek letter (λ) or a backslash (\\ -
 /// less aesthetic, but only one byte in size)
-/// - the identifiers in `Classic` mode are `String`s of ASCII alphabetic characters
-/// - `Classic` mode ignores whitespaces where unambiguous
-/// - the indices in the `DeBruijn` notation mode start with 1 and are hexadecimal digits
-/// - `DeBruijn` mode ignores all whitespaces (since indices > 15 are very unlikely)
+/// - the identifiers in `Classic` notation are `String`s of ASCII alphabetic characters
+/// - `Classic` notation ignores whitespaces where unambiguous
+/// - the indices in the `DeBruijn` notation start with 1 and are hexadecimal digits
+/// - `DeBruijn` notation ignores all whitespaces (since indices > 15 are very unlikely)
 ///
-/// # Example
+/// # Examples
 /// ```
-/// use lambda_calculus::parser::*;
-/// use lambda_calculus::combinators::{s, y};
+/// use lambda_calculus::*;
+/// use lambda_calculus::combinators::{S, Y};
 ///
-/// assert_eq!(parse(&"λf.(λx.f (x x)) (λx.f (x x))", Classic), Ok(y()));
-/// assert_eq!(parse(&"λf.(λx.f(x x))(λx.f(x x))", Classic),    Ok(y()));
+/// assert_eq!(parse(&"λf.(λx.f (x x)) (λx.f (x x))", Classic), Ok(Y()));
+/// assert_eq!(parse(&"λf.(λx.f(x x))(λx.f(x x))", Classic),    Ok(Y()));
 ///
-/// assert_eq!(parse(  &"λλλ31(21)",     DeBruijn), Ok(s()));
-/// assert_eq!(parse(&r#"\\\3 1 (2 1)"#, DeBruijn), Ok(s()));
+/// assert_eq!(parse(  &"λλλ31(21)",     DeBruijn), Ok(S()));
+/// assert_eq!(parse(&r#"\\\3 1 (2 1)"#, DeBruijn), Ok(S()));
 /// ```
-pub fn parse(input: &str, notation: Notation) -> Result<Term, Error> {
+///
+/// # Errors
+///
+/// Returns a `ParseError` when a lexing or syntax error is encountered.
+pub fn parse(input: &str, notation: Notation) -> Result<Term, ParseError> {
     let tokens = if notation == DeBruijn {
         tokenize_dbr(input)?
     } else {
@@ -240,7 +244,7 @@ pub fn parse(input: &str, notation: Notation) -> Result<Term, Error> {
 }
 
 #[doc(hidden)]
-pub fn fold_exprs(exprs: &[Expression]) -> Result<Term, Error> {
+pub fn fold_exprs(exprs: &[Expression]) -> Result<Term, ParseError> {
     let mut depth  = 0;
     let mut output = Vec::new();
 
@@ -264,7 +268,7 @@ pub fn fold_exprs(exprs: &[Expression]) -> Result<Term, Error> {
     Ok(ret)
 }
 
-fn fold_terms(mut terms: Vec<Term>) -> Result<Term, Error> {
+fn fold_terms(mut terms: Vec<Term>) -> Result<Term, ParseError> {
     if terms.len() > 1 {
         let fst = terms.remove(0);
         Ok( terms.into_iter().fold(fst, app) )
@@ -281,8 +285,8 @@ mod tests {
 
     #[test]
     fn tokenization_error() {
-        assert_eq!(tokenize_dbr(&"λλx2"),    Err(InvalidCharacter((3, 'x'))));
-        assert_eq!(tokenize_cla(&"λa.λb a"), Err(InvalidCharacter((6, ' '))));
+        assert_eq!(tokenize_dbr(&"λλx2"),    Err(InvalidCharacter((2, 'x'))));
+        assert_eq!(tokenize_cla(&"λa.λb a"), Err(InvalidCharacter((5, ' '))));
     }
 
     #[test]
